/*
	IntToFloatCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include "IntToFloatCmd.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


IntToFloatCmd::IntToFloatCmd ( char *name, int active, char *intFile, char *floatFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, intFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, floatFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_percentDone = 0;
	_error = 0;
	_offset = 0;
	_scale = 1;
}


IntToFloatCmd::~IntToFloatCmd()
{	
	// empty
}
	

void IntToFloatCmd::doCommand()
{
	if ( _done == True )
		return;
	
	int percent;
	char text[5];
	int error = 0;
	int numElements = 1000;

	if ( !_once )
	{
		// can we open source for read?
		FileAccess *src = new FileAccess ( _srcFile, "r" );
		
		if ( src->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, src->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		delete src;
		
		// can we open dst for write?
		FileAccess *dst = new FileAccess ( _dstFile, "w" );
		
		if ( dst->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nmFile write error: %s", _dstFile, dst->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		delete dst;
		
		if ( error )
		{
			_done = True;
			_error = error;
			return;
		}
		
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		FileAccess *out = new FileAccess ( _dstFile, "w"  );
		
		_in = in;
		_out = out;

		_in->elementSize ( sizeof ( int ) );
		_out->elementSize ( sizeof ( float ) );
		_in->numElements ( 1 );
		_out->numElements ( 1 );

		updateTimeout ( ( int ) ( _in->fileSize() / ( numElements * sizeof ( int ) ) ) + 1 );
		_once = 1;
	}
	

	if ( numElements > ( int ) ( ( _in->fileSize() - _in->filePos() ) / sizeof ( int ) ) )
		numElements = ( _in->fileSize() - _in->filePos() ) / sizeof ( int );
	
	if ( _in->filePos() < _in->fileSize() )
	{
		while ( numElements > 0 )
		{
			_in->readF ( &_iValue );
			_fValue = ( float ) ( ( _iValue + _offset ) * _scale );
			_out->writeF ( &_fValue );
			numElements--;
		}

		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}
	}

	if ( _in->filePos() == _in->fileSize() )
	{
		delete _in;
		delete _out;
		_done = True;
	}
}

