/*
    SplitWaveCmd.cc

    Will Eighteen, 2000: will@opaline.demon.co.uk
*/                                                                             

#include "SplitWaveCmd.h"
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


SplitWaveCmd::SplitWaveCmd ( char *name, int active, char *wavFile, long blockLength) : BreakCmd ( name, active )
{
	char baseName[BUFSIZE];
	unsigned int baseLength = 0;

	_blockLength = blockLength;
	_once = 0;
	_percentDone = 0;
	_error = 0;
	strncpy ( _srcFile, wavFile, sizeof ( _srcFile ) );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	baseLength = strstr ( _srcFile, ".wav" ) - _srcFile;
	
	if ( baseLength > ( sizeof ( baseName ) - 1 ) )
		baseLength = sizeof ( baseName ) - 1;
		
	strncpy ( baseName, _srcFile, baseLength );
	baseName [ baseLength ] = 0;

	strncpy ( _dstFileL, baseName, sizeof ( _dstFileL ) );
	_dstFileL [ sizeof ( _dstFileL ) - 1 ] = 0;
	strncat ( _dstFileL, ".chL.wav", sizeof ( _dstFileL ) - strlen ( _dstFileL ) - 1 );

	strncpy ( _dstFileR, baseName, sizeof ( _dstFileR ) );
	_dstFileR [ sizeof ( _dstFileR ) - 1 ] = 0;
	strncat ( _dstFileR, ".chR.wav", sizeof ( _dstFileR ) - strlen ( _dstFileR ) - 1 );
}


SplitWaveCmd::~SplitWaveCmd()
{	
	// empty
}
	

void SplitWaveCmd::doCommand()
{
	if ( _done == True )
		return;
	
	int percent;
	char text[5];
	int error = 0;
	int count;

	if ( !_once )
	{
		_pWave = new Wave;
		_pLeft = new Wave;
		_pRight = new Wave;

		if ( !_pWave->LoadHeader ( _srcFile ) )
		{
			char msg[BUFSIZE];

    		strncpy ( msg, _pWave->name(), sizeof ( msg ) );
			msg [ sizeof( msg ) - 1 ] = 0;
			strncat ( msg, "\nWave::SaveHeader failed.", sizeof ( msg ) - strlen ( msg ) - 1 );
			messageDialogManager->post ( msg );
			error++;
		}

		if ( _pWave->numChannels() != 2 )
		{
			char msg[BUFSIZE];

			snprintf ( msg, sizeof ( msg ), "%s\nNot a stereo file: need 2 channels, found %d.",
						_pWave->name(), _pWave->numChannels() );
			
			msg [ sizeof ( msg ) - 1 ] = 0;				
			messageDialogManager->post ( msg );
			error++;
		}

		if ( _blockLength < ( int ) ( _pWave->bitsPerSample() / 8 ) )
		{
			char msg[BUFSIZE];

			snprintf ( msg, sizeof ( msg ), "%s\nSplit wave: buffer size %ld insufficient.", _pWave->name(), _blockLength );
			msg [ sizeof ( msg ) - 1 ] = 0;								
			messageDialogManager->post ( msg );
			error++;
		}

		if ( error )
		{
			_done = True;
			_error = error;	// return here to avoid writing output headers
			delete _pWave;
			delete _pLeft;
			delete _pRight;
			return;
		}
		
		error = 0;
		// can we open dstL,R for write?
		_pLeft->FillHeader ( 1, _pWave->sampleRate(), _pWave->bitsPerSample(), 0 );

		if ( !_pLeft->SaveHeader ( _dstFileL ) )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nmSaveHeader failed.", _dstFileL );
			msg [ sizeof ( msg ) - 1 ] = 0;								
			messageDialogManager->post ( msg );
		}
			
		_pRight->FillHeader ( 1, _pWave->sampleRate(), _pWave->bitsPerSample(), 0 );
	
		if ( !_pRight->SaveHeader ( _dstFileR ) )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nSaveHeader failed.", _dstFileR );
			msg [ sizeof ( msg ) - 1 ] = 0;								
			messageDialogManager->post ( msg );
		}
		
		if ( error )
		{
			_done = True;
			_error = error;
			delete _pWave;
			delete _pLeft;
			delete _pRight;
			return;
		}
		
		_pWave->filePos = 0;
		updateTimeout ( ( int ) ( _pWave->dataLength() / _blockLength ) + 1 );
		_once = 1;
	}

	if ( _blockLength > ( _pWave->dataLength() - _pWave->filePos ) )
		_blockLength = _pWave->dataLength() - _pWave->filePos;

	char *lData;
	char *rData;
	char *oData;
		
	count = 0;

	_pLeft->AllocateDataSpace ( _blockLength );	// mono out >>1?
	lData = _pLeft->data();
	_pRight->AllocateDataSpace ( _blockLength );
	rData = _pRight->data();

	_pWave->LoadBlock ( _pWave->filePos, _blockLength );
	oData = _pWave->data();

// specifically 16-bit, sigh
	while ( count < _blockLength>>1 )
	{
		lData[count] = oData[2*count];
		lData[count+1] = oData[(2*count)+1];

		rData[count] = oData[(2*count)+2];
		rData[count+1] = oData[(2*count)+3];

		count += 2;
	}

	_pWave->filePos += _blockLength;
		
	_pLeft->FillHeader ( 1, _pWave->sampleRate(), _pWave->bitsPerSample(), _pWave->filePos>>2 );
	_pLeft->AppendBlock ( _dstFileL,1, _pWave->sampleRate(), _pWave->bitsPerSample(), _blockLength>>2 );
	_pLeft->ChangeHeader ( _dstFileL );      

	_pRight->FillHeader ( 1, _pWave->sampleRate(), _pWave->bitsPerSample(), _pWave->filePos>>2);
	_pRight->AppendBlock ( _dstFileR, 1, _pWave->sampleRate(), _pWave->bitsPerSample(), _blockLength>>2 );
	_pRight->ChangeHeader ( _dstFileR );
		
	percent = ( int ) ( ( float ) _pWave->filePos / ( float ) _pWave->dataLength() * 100 );

	if ( _percentDone != percent )
	{
		_percentDone = percent;
		snprintf ( text, 5, "%d%%", _percentDone );
		updateMessage ( text );
	}

	if ( _pWave->filePos == _pWave->dataLength() )
	{
		delete _pWave;
		delete _pLeft;
		delete _pRight;
		_done = True;
	}
	else
	{	// deallocate all but final block (freed by wave deletion)
		_pLeft->Free();
		_pRight->Free();
	}
}

